'                                 SORTDEMO
' Dieses Programm demonstriert grafisch sechs gebruchliche Sortier-
' algorithmen. Es gibt 25 oder 43 horizontale Balken aus, die alle eine
' unterschiedliche Lnge haben, in zuflliger Reihenfolge vorliegen und
' der Lnge nach sortiert werden.
'
' Auerdem verwendet das Programm SOUND-Anweisungen, um abhngig von
' der Position des auszugebenden Balkens unterschiedliche Tonhhen zu
' erzeugen. Beachten Sie, da die SOUND-Anweisungen die Geschwindigkeit
' jedes Sortieralgorithmus' verlangsamen, so da Sie den Fortgang der
' Sortierung verfolgen knnen. Daher sind die gezeigten Zeiten nur zum
' Vergleich bestimmt. Sie sind keine genaue Messung der Sortier-
' geschwindigkeit.
'
' Wenn Sie diese Sortierroutinen in eigenen Programmen verwenden, werden
' Sie vielleicht Unterschiede in deren relativen Geschwindigkeiten fest-
' stellen (zum Beispiel kann das Sortieren durch Austausch schneller sein
' als der Shellsort), abhngig von der Anzahl der zu sortierenden Elemente
' und wie "ungeordnet" diese zu Beginn des Sortiervorgangs vorliegen.
' ============================================================================

DEFINT A-Z      ' Standardtyp ist Ganzzahl.

' Deklariere FUNCTION- und SUB-Prozeduren sowie Anzahl und Typ der Argumente:
  DECLARE FUNCTION ZufInt% (Untere, Obere)

  DECLARE SUB FensterInit ()
  DECLARE SUB BubbleSort ()
  DECLARE SUB PruefMonit ()
  DECLARE SUB ZeichnRahmen (SeiteOben, SeiteUnten, SeiteLinks, SeiteRechts)
  DECLARE SUB VerbrauchZeit (AktZeile)
  DECLARE SUB AustauschSort ()
  DECLARE SUB HeapSort ()
  DECLARE SUB Initialisiere ()
  DECLARE SUB EinfuegeSort ()
  DECLARE SUB FilternAbw (MaxEbene)
  DECLARE SUB FilternAufw (MaxEbene)
  DECLARE SUB ZeichneBalken (Zeile)
  DECLARE SUB QuickSort (Klein, Gross)
  DECLARE SUB ReInitialisiere ()
  DECLARE SUB ShellSort ()
  DECLARE SUB SortMenue ()
  DECLARE SUB TauscheBalken (Zeile1, Zeile2)
  DECLARE SUB UmschaltMus (Zeile, Spalte)

' Definiere den Datentyp, der dazu verwendet wird, Informationen zu jedem
' farbigen Balken aufzunehmen:
  TYPE SortType
     Laenge AS INTEGER          ' Balkenlnge (das in den unterschiedlichen
                                ' Sortierungen zu vergleichende Element)
     FarbWert AS INTEGER        ' Balkenfarbe
     BalkZeichK AS STRING * 43  ' Der Balken (eine Zeichenkette mit 43
  END TYPE                      ' Zeichen)

' Deklariere globale Konstanten:
  CONST FALSCH = 0, WAHR = NOT FALSCH, SPALTELINKS = 49
  CONST ANZOPTIONEN = 11, ANZSORT = 6

' Deklariere globale Variablen und weise diesen Speicherplatz zu. Sowohl
' SortDatFeld als auch SortSicher sind Datenfelder, die den oben definierten
' Datentyp SortType erhalten:
  DIM SHARED SortDatFeld(1 TO 43) AS SortType, SortSicher(1 TO 43) AS SortType
  DIM SHARED OptionNamen(1 TO ANZOPTIONEN) AS STRING * 13
  DIM SHARED StartZeit AS SINGLE
  DIM SHARED Vordergrund, Hintergrund, KeineMus, Pause
  DIM SHARED Wahl, MaxZeile, InitZeile, MaxFarben

' DATA-Anweisungen fr die unterschiedlichen Optionen, die in dem Sortiermen
' ausgegeben werden:
  DATA Einfge, Bubble, Heap, Austausch, Shell, Quick,
  DATA Musik ein/aus, , < (Langsamer), > (Schneller)

' Logischer Beginn des Modul-Ebenen-Codes:

  Initialisiere             ' Initialisiere Datenwerte.
  SortMenue                 ' Gibt Sortiermen aus.
  WIDTH 80, InitZeile       ' Stelle ursprngliche Anzahl der Zeilen
  CLS                       ' wieder her.
  END

' LiesZeile, MonoVerf und ZeileVerf sind Routinen zur Fehlerbehandlung,
' die von der SUB-Prozedur PruefMonit aufgerufen werden. LiesZeile ermittelt,
' ob das Programm mit 25, 43 oder 50 Zeilen gestartet wird. MonoVerf legt
' fest, da der aktuelle Video-Adapter monochrom ist. ZeileVerf setzt die
' maximal mgliche Anzahl an Zeilen (43 oder 25).

LiesZeile:
   IF InitZeile = 50 THEN
      InitZeile = 43
      RESUME
   ELSE
      InitZeile = 25
      RESUME NEXT
   END IF

MonoVerf:
   MaxFarben = 2
   RESUME NEXT

ZeileVerf:
   MaxZeile = 25
   RESUME

' ============================ AustauschSort ================================
'   Der Algorithmus "Sortieren durch Austauschen" vergleicht jedes Element
'   in SortDatFeld - beginnend mit dem ersten Element - mit jedem folgenden
'   Element. Wenn eines der nachfolgenden Elemente kleiner ist als das
'   aktuelle Element, wird es mit dem aktuellen Element getauscht, und der
'   Ablauf wird mit dem nchsten Element in SortDatFeld wiederholt.
' ==========================================================================
'
SUB AustauschSort STATIC
   FOR Zeile = 1 TO MaxZeile
      KleinstZeile = Zeile
      FOR J = Zeile + 1 TO MaxZeile
         IF SortDatFeld(J).Laenge < SortDatFeld(KleinstZeile).Laenge THEN
            KleinstZeile = J
            VerbrauchZeit J
         END IF
      NEXT J

      ' Es ist eine Zeile gefunden, die kleiner als die aktuelle Zeile
      ' ist, also vertausche diese beiden Datenfeldelemente:
      IF KleinstZeile > Zeile THEN
         SWAP SortDatFeld(Zeile), SortDatFeld(KleinstZeile)
         TauscheBalken Zeile, KleinstZeile
      END IF
   NEXT Zeile
END SUB

' ============================ BubbleSort ================================
'    Der "BubbleSort"-Algorithmus durchluft SortDatFeld, vergleicht auf-
'    einanderfolgende Elemente und vertauscht Paare, die nicht in der
'    richtigen Reihenfolge vorliegen. Er fhrt damit fort, bis keine
'    Paare mehr getauscht wurden.
' =========================================================================
'
SUB BubbleSort STATIC
   Limit = MaxZeile
   DO
      Tauschen = FALSCH
      FOR Zeile = 1 TO (Limit - 1)

         ' Zwei aufeinanderfolgende Elemente liegen nicht in der richtigen
         ' Reihenfolge vor, also tausche deren Werte und zeichne ihre
         ' Balken neu:
         IF SortDatFeld(Zeile).Laenge > SortDatFeld(Zeile + 1).Laenge THEN
            SWAP SortDatFeld(Zeile), SortDatFeld(Zeile + 1)
            TauscheBalken Zeile, Zeile + 1
            Tauschen = Zeile
         END IF
      NEXT Zeile

      ' Sortiere im nchsten Schritt nur bis dahin, wo der letzte Tausch
      ' vorgenommen wurde:
      Limit = Tauschen
   LOOP WHILE Tauschen

END SUB

' ============================= EinfuegeSort ================================
'   Die Prozedur EinfuegeSort vergleicht nacheinander die Lnge jedes
'   Elementes in SortDatFeld mit der Lnge aller vorhergehenden Elemente.
'   Nachdem die Prozedur die entsprechende Position fr das neue Element
'   gefunden hat, fgt es das Element an seinem neuen Platz ein und bewegt
'   alle anderen Elemente um eine Position nach unten.
' ===========================================================================
'
SUB EinfuegeSort STATIC
   DIM TempWert AS SortType
   FOR Zeile = 2 TO MaxZeile
      TempWert = SortDatFeld(Zeile)
      TempLaenge = TempWert.Laenge
      FOR J = Zeile TO 2 STEP -1

         ' Solange die Lnge des J-1sten Elementes grer als die Lnge des
         ' ursprnglichen Elementes in SortDatFeld(Zeile) ist, fahre fort,
         ' die Datenfeldelemente nach unten zu verschieben:
         IF SortDatFeld(J - 1).Laenge > TempLaenge THEN
            SortDatFeld(J) = SortDatFeld(J - 1)
            ZeichneBalken J       ' Gib den neuen Balken aus.
            VerbrauchZeit J       ' Gib die verbrauchte Zeit aus.

         ' Andernfalls beende die FOR...NEXT-Schleife:
         ELSE
            EXIT FOR
         END IF
      NEXT J

      ' Fge den ursprnglichen Wert von SortDatFeld(Zeile) in
      ' SortDatFeld(J) ein:
      SortDatFeld(J) = TempWert
      ZeichneBalken J
      VerbrauchZeit J
   NEXT Zeile
END SUB

' =========================== FensterInit ==================================
'   Ruft die Prozedur ZeichnRahmen auf, die den Rahmen um das Sortiermen
'   herum zeichnet, und gibt anschlieend die unterschiedlichen Optionen
'   aus, die in dem Datenfeld OptionNamen gespeichert sind.
' ============================================================================
'
SUB FensterInit STATIC
   ZeichnRahmen 1, 22, SPALTELINKS - 3, 78

   LOCATE 3, SPALTELINKS + 2: PRINT "QUICKBASIC SORTIER-DEMO";
   LOCATE 5
   FOR I = 1 TO ANZOPTIONEN - 1
      LOCATE , SPALTELINKS: PRINT OptionNamen(I)
   NEXT I

   ' Gibt die letzte Option (> Schneller) nicht aus, falls die Pause
   ' ein Uhrtakt lang ist:
   IF Pause > 1 THEN LOCATE , SPALTELINKS: PRINT OptionNamen(ANZOPTIONEN);

   ' Schalte Musik ein oder aus und gib anschlieend den aktuellen Wert
   ' fr KeineMus aus:
   KeineMus = NOT KeineMus
   UmschaltMus 12, SPALTELINKS + 13

   LOCATE ANZOPTIONEN + 6, SPALTELINKS
   PRINT "Geben Sie das erste Zeichen"
   LOCATE , SPALTELINKS
   PRINT "des gewhlten Verfahrens"
   LOCATE , SPALTELINKS
   PRINT "( E B H A S Q M < > )"
   LOCATE , SPALTELINKS
   PRINT "oder ESC zum Beenden ein: ";
END SUB

' ============================ FilternAbw =================================
'   Die Prozedur FilternAbw erzeugt mit den Elementen aus SortDatFeld von
'   1 bis MaxEbene erneut einen "Heap" (siehe das Diagramm in der Prozedur
'   HeapSort).
' ============================================================================
'
SUB FilternAbw (MaxEbene) STATIC
   I = 1

   ' Bewege den Wert in SortDatFeld(1) im Heap solange nach unten, bis
   ' dieser seinen richtigen Knoten erreicht hat (das heit, bis der Wert
   ' kleiner als sein Eltern-Knoten ist, oder bis er MaxEbene, die unterste
   ' Ebene des aktuellen Heaps, erreicht hat):
   DO
      Kind = 2 * I        ' Ermittle den Index fr den Kind-Knoten.

      ' Verlasse die Prozedur, wenn unterste Ebene des Heaps erreicht ist:
      IF Kind > MaxEbene THEN EXIT DO

      ' Falls es zwei Kind-Knoten gibt, finde heraus, welcher der grere ist:
      IF Kind + 1 <= MaxEbene THEN
         IF SortDatFeld(Kind + 1).Laenge > SortDatFeld(Kind).Laenge THEN
            Kind = Kind + 1
         END IF
      END IF

      ' Bewege den Wert nach unten, solange er noch nicht grer als
      ' irgendeines seiner Kinder ist:
      IF SortDatFeld(I).Laenge < SortDatFeld(Kind).Laenge THEN
         SWAP SortDatFeld(I), SortDatFeld(Kind)
         TauscheBalken I, Kind
         I = Kind

      ' Andernfalls ist SortDatFeld erneut als Heap von 1 bis MaxEbene
      ' aufgebaut, also beende:
      ELSE
         EXIT DO
      END IF
   LOOP
END SUB

' ============================== FilternAufw =================================
'   Die Prozedur FilternAufw bertrgt die Elemente von 1 bis MaxEbene in
'   SortDatFeld in einen "heap" (siehe das Diagramm in der Prozedur HeapSort).
' ============================================================================
'
SUB FilternAufw (MaxEbene) STATIC
   I = MaxEbene

   ' Bewege den Wert in SortDatFeld(MaxEbene) solange durch den Heap nach
   ' oben, bis er seinen richtigen Knoten erreicht hat (das heit, bis der
   ' Wert grer als irgendeiner seiner Kind-Knoten ist, oder er 1, die
   ' Spitze des Heaps, erreicht hat):
   DO UNTIL I = 1
      Eltern = I \ 2        ' Lies den Index des Eltern-Knotens.

      ' Der Wert des aktuellen Knotens ist noch grer als der Wert seines
      ' Eltern-Knotens, also vertausche diese beiden Datenfeldelemente:
      IF SortDatFeld(I).Laenge > SortDatFeld(Eltern).Laenge THEN
         SWAP SortDatFeld(Eltern), SortDatFeld(I)
         TauscheBalken Eltern, I
         I = Eltern

      ' Andernfalls hat das Element in dem Heap seine richtige Position
      ' erreicht, also verlasse diese Prozedur:
      ELSE
         EXIT DO
      END IF
   LOOP
END SUB

' =============================== HeapSort ===================================
'  Die Prozedur HeapSort funktioniert, indem sie zwei andere Prozeduren
'  aufruft - FilternAufw und FilternAbw. FilternAufw wandelt SortDatFeld
'  in einen "heap" um, dessen Eigenschaften das unten gezeigte Diagramm
'  verdeutlicht:
'
'                           SortDatFeld(1)
'                           /            \
'              SortDatFeld(2)            SortDatFeld(3)
'               /           \            /            \
'       SortDatFeld(4) SortDatFeld(5) SortDatFeld(6) SortDatFeld(7)
'          /      \       /       \       /      \      /      \
'        ...      ...   ...       ...   ...      ...  ...      ...
'
'
'  wobei jeder "Eltern-Knoten" grer ist als jeder seiner "Kind-Knoten";
'  zum Beispiel ist SortDatFeld(1) grer als SortDatFeld(2) oder
'  SortDatFeld(3), SortDatFeld(3) ist grer als SortDatFeld(6) oder
'  SortDatFeld(7) und so weiter.
'
'  Nachdem die erste FOR...NEXT-Schleife in HeapSort beendet ist, befindet
'  sich das grte Element daher in SortDatFeld(1).
'
'  Die zweite FOR...NEXT-Schleife in HeapSort vertauscht das Element in
'  SortDatFeld(1) mit dem Element in MaxZeile, bildet den Heap erneut (mit
'  FilternAbw) fr MaxZeile - 1, vertauscht anschlieend das Element in
'  SortDatFeld(1) mit dem Element in MaxZeile - 1, bildet den Heap erneut
'  fr MaxZeile - 2 und fhrt in dieser Art und Weise fort, bis das Datenfeld
'  sortiert ist.
' ============================================================================
'
SUB HeapSort STATIC
   FOR I = 2 TO MaxZeile
      FilternAufw I
   NEXT I

   FOR I = MaxZeile TO 2 STEP -1
      SWAP SortDatFeld(1), SortDatFeld(I)
      TauscheBalken 1, I
      FilternAbw I - 1
   NEXT I
END SUB

' ============================ Initialisiere ================================
'    Initialisiert die Datenfelder SortSicher und OptionNamen. Ruft
'    auerdem die Prozeduren PruefMonit, FensterInit und ZufInt% auf.
' ===========================================================================
'
SUB Initialisiere STATIC
   DIM TempDatFeld(1 TO 43)

   PruefMonit              ' Prfe auf monochrom oder EGA und setze
                           ' maximale Anzahl an Textzeilen.
   FOR I = 1 TO MaxZeile
      TempDatFeld(I) = I
   NEXT I

   MaxIndex = MaxZeile

   RANDOMIZE TIMER         ' Ausgangspunkt des Zufallszahlen-Generators.
   FOR I = 1 TO MaxZeile

      ' Rufe ZufInt% auf, um in TempDatFeld ein Zufallselement zwischen 1
      ' und MaxIndex zu finden, weise anschlieend den Wert dieses Elementes
      ' BalkLaenge zu:
      Index = ZufInt%(1, MaxIndex)
      BalkLaenge = TempDatFeld(Index)

      ' berschreibe den Wert in TempDatFeld(Index) mit dem Wert in
      ' TempDatFeld(MaxIndex), so da der Wert in TempDatFeld(Index) nur
      ' einmal gewhlt ist:
      TempDatFeld(Index) = TempDatFeld(MaxIndex)

      ' Verkleinere den Wert von MaxIndex, so da TempDatFeld(MaxIndex)
      ' beim nchsten Schritt durch die Schleife nicht gewhlt werden kann:
      MaxIndex = MaxIndex - 1

      ' Weise den Wert von BalkLaenge dem Element .Laenge zu und speichere
      ' anschlieend eine Zeichenkette mit BalkLaenge Balkenzeichen (ASCII
      ' 223: ) in dem Element .BalkZeichK:
      SortSicher(I).Laenge = BalkLaenge
      SortSicher(I).BalkZeichK = STRING$(BalkLaenge, 223)

      ' Speichere in dem Element .FarbWert den passenden Farbwert:
      IF MaxFarben > 2 THEN
         SortSicher(I).FarbWert = (BalkLaenge MOD MaxFarben) + 1
      ELSE
         SortSicher(I).FarbWert = MaxFarben
      END IF
   NEXT I

   FOR I = 1 TO ANZOPTIONEN      ' Lies die Men-Optionen von SORT-DEMO
                                 ' und speichere diese in dem Datenfeld
      READ OptionNamen(I)        ' OptionNamen.
   NEXT I

   CLS
   ReInitialisiere   ' Weise die Werte in SortSicher SortDatFeld zu und
                     ' zeichne unsortierte Balken auf den Bildschirm.
   KeineMus = FALSCH
   Pause = 2         ' Initialisiere Pause auf 2 Uhrtakte (@ 1/9 Sekunden).
   FensterInit       ' Zeichne Rahmen um das Sortiermen und gib Optionen aus.

END SUB

' ============================= PruefMonit ================================
'    Ermittelt den Monitor-Typ (VGA, EGA, CGA oder monochrom) sowie die
'    anfngliche Anzahl der Bildschirmzeilen (50, 43 oder 25).
' ==========================================================================
'
SUB PruefMonit STATIC

   ' Versuche, in der 50-ten Zeile zu positionieren; wenn dies nicht gelingt,
   ' versuche die 43-ste. Wenn auch dies nicht gelingt, verwendet der Benutzer
   ' den 25-Zeilen-Modus:
   InitZeile = 50
   ON ERROR GOTO LiesZeile
   LOCATE InitZeile, 1

   ' Versuche eine Anweisung SCREEN 1, um zu berprfen, ob der aktuelle
   ' Adapter Farbgrafik ermglicht; wenn dies zu einem Fehler fhrt, setze
   ' MaxFarben auf 2 zurck:
   MaxFarben = 15
   ON ERROR GOTO MonoVerf
   SCREEN 1
   SCREEN 0

   ' berprfe, ob 43-Zeilen-Modus akzeptiert wird; wenn nicht, fhre dieses
   ' Programm im 25-Zeilen-Modus aus:
   MaxZeile = 43
   ON ERROR GOTO ZeileVerf
   WIDTH 80, MaxZeile
   ON ERROR GOTO 0              ' Schalte Fehlerverfolgung aus.
END SUB

' ============================== QuickSort ===================================
'   Der "QuickSort"-Algorithmus funktioniert, indem er ein zuflliges "Pivot"-
'   Element aus SortDatFeld herausnimmt, anschlieend jedes Element, das
'   grer ist, auf eine Seite des Pivot-Elementes bewegt, und jedes Element,
'   das kleiner ist, auf die andere Seite bewegt. QuickSort wird dann mit den
'   beiden Unterabteilungen, die von dem Pivot-Element erzeugt wurden,
'   rekursiv aufgerufen. Nachdem die Anzahl der Elemente in einer Unter-
'   abteilung einmal zwei erreicht hat, enden die rekursiven Aufrufe, und das
'   Datenfeld ist sortiert.
' ============================================================================
'
SUB QuickSort (Klein, Gross)
   IF Klein < Gross THEN

      ' Nur zwei Elemente in dieser Unterabteilung; vertausche diese, wenn
      ' sie nicht in der richtigen Reihenfolge vorliegen und beende
      ' anschlieend die rekursiven Aufrufe:
      IF Gross - Klein = 1 THEN
         IF SortDatFeld(Klein).Laenge > SortDatFeld(Gross).Laenge THEN
            SWAP SortDatFeld(Klein), SortDatFeld(Gross)
            TauscheBalken Klein, Gross
         END IF
      ELSE

         ' Nimm ein zuflliges Pivot-Element heraus, bewege dieses dann an
         ' das Ende:
         ZufIndex = ZufInt%(Klein, Gross)
         SWAP SortDatFeld(Gross), SortDatFeld(ZufIndex)
         TauscheBalken Gross, ZufIndex
         TeilStck = SortDatFeld(Gross).Laenge
         DO

            ' Hinbewegung von beiden Seiten auf das Pivot-Element zu:
            I = Klein: J = Gross
            DO WHILE (I < J) AND (SortDatFeld(I).Laenge <= TeilStck)
               I = I + 1
            LOOP
            DO WHILE (J > I) AND (SortDatFeld(J).Laenge >= TeilStck)
               J = J - 1
            LOOP

            ' Wird das Pivot-Element nicht erreicht, bedeutet dies, da
            ' zwei Elemente auf einer Seite nicht in der richtigen Reihenfolge
            ' vorliegen, also vertausche diese Elemente:
            IF I < J THEN
               SWAP SortDatFeld(I), SortDatFeld(J)
               TauscheBalken I, J
            END IF
         LOOP WHILE I < J

         ' Bewege das Pivot-Element zurck auf seinen richtigen Platz in
         ' dem Datenfeld:
         SWAP SortDatFeld(I), SortDatFeld(Gross)
         TauscheBalken I, Gross

         ' Rufe die Prozedur QuickSort rekursiv auf (bergib die kleinere
         ' Unterabteilung zuerst, um weniger Stapelplatz zu verwenden):
         IF (I - Klein) < (Gross - I) THEN
            QuickSort Klein, I - 1
            QuickSort I + 1, Gross
         ELSE
            QuickSort I + 1, Gross
            QuickSort Klein, I - 1
         END IF
      END IF
   END IF
END SUB

' ============================ ReInitialisiere ==============================
'   berfhrt das Datenfeld SortDatFeld wieder in seinen ursprnglichen
'   unsortierten Zustand und gibt anschlieend die unsortierten Farbbalken
'   aus.
' ===========================================================================
'
SUB ReInitialisiere STATIC
   FOR I = 1 TO MaxZeile
      SortDatFeld(I) = SortSicher(I)
   NEXT I

   FOR I = 1 TO MaxZeile
      LOCATE I, 1
      COLOR SortDatFeld(I).FarbWert
      PRINT SortDatFeld(I).BalkZeichK;
   NEXT I

   COLOR MaxFarben, 0
END SUB

' =============================== ShellSort ==================================
'  Die Prozedur ShellSort ist hnlich zu der Prozedur BubbleSort. ShellSort
'  startet jedoch damit, da sie weit auseinanderliegende Elemente vergleicht
'  (getrennt durch den Wert der Variablen Offset, der zu Beginn die Hlfte des
'  Abstandes zwischen dem ersten und letzten Element ist) und anschlieend
'  Elemente vergleicht, die nher zusammenliegen (wenn Offset eins ist, ist
'  die letzte Iteration dieser Prozedur gleich der Prozedur BubbleSort).
' ============================================================================
'
SUB ShellSort STATIC

   ' Setze den Vergleichsoffset auf die Hlfte der Satzzahl in SortDatFeld:
   Offset = MaxZeile \ 2

   DO WHILE Offset > 0              ' Wiederhole, bis Offset null wird.
      Limit = MaxZeile - Offset
      DO
         Tauschen = FALSCH          ' Nimm an, da bei diesem Offset nicht
                                    ' getauscht wird.
         ' Vergleiche die Elemente und vertausche diejenigen, die nicht in der
         ' richtigen Reihenfolge liegen:
         FOR Zeile = 1 TO Limit
            IF SortDatFeld(Zeile).Laenge > SortDatFeld(Zeile + Offset).Laenge THEN
               SWAP SortDatFeld(Zeile), SortDatFeld(Zeile + Offset)
               TauscheBalken Zeile, Zeile + Offset
               Tauschen = Zeile
            END IF
         NEXT Zeile

         ' Sortiere im nchsten Schritt nur bis dahin, wo der letzte Tausch
         ' durchgefhrt wurde:
         Limit = Tauschen - Offset
      LOOP WHILE Tauschen

      ' Kein Tausch beim letzten Offset, versuche es mit dem halbierten
      ' Offset:
      Offset = Offset \ 2
   LOOP
END SUB

' ============================= SortMenue =================================
'   Die Prozedur SortMenue ruft zuerst die Prozedur ReInitialisiere auf um
'   sicherzustellen, da SortDatFeld in unsortierter Form vorliegt und
'   fordert den Benutzer danach auf, zwischen den folgenden Mglichkeiten
'   zu whlen:
'
'               * Einen der Sortier-Algorithmen
'               * Musik ein- bzw. ausschalten
'               * Geschwindigkeit erhhen bzw. vermindern
'               * Das Programm beenden
' =========================================================================
'
SUB SortMenue STATIC
   Escape$ = CHR$(27)

   ' Bilde eine Zeichenkette, die aus allen zulssigen Wahlmglichkeiten
   ' besteht:
   Option$ = "EBHASQ><M" + Escape$

   DO

      ' Lasse den Cursor sichtbar werden:
      LOCATE ANZOPTIONEN + 8, SPALTELINKS + 27, 1

      Auswahl$ = UCASE$(INPUT$(1))    ' Lies die Wahl des Benutzers und
      Wahl = INSTR(Option$, Auswahl$) ' berprfe, ob sie eine der
                                      ' Menoptionen ist.

      ' Der Benutzer whlte eine der Sortierprozeduren:
      IF (Wahl >= 1) AND (Wahl <= ANZSORT) THEN
         ReInitialisiere              ' Mische die Balken erneut.
         LOCATE , , 0                 ' Lasse den Cursor unsichtbar werden.
         Vordergrund = 0              ' Setze Werte fr invertierte
         Hintergrund = 7              ' Darstellung.
         StartZeit = TIMER            ' Halte die Startzeit fest.
      END IF

      ' Verzweige abhngig von der bettigten Taste zu der entsprechenden
      ' Prozedur:
      SELECT CASE Auswahl$
         CASE "E"
            EinfuegeSort
         CASE "B"
            BubbleSort
         CASE "H"
            HeapSort
         CASE "A"
            AustauschSort
         CASE "S"
            ShellSort
         CASE "Q"
            QuickSort 1, MaxZeile
         CASE ">"

            ' Verkrze die Pausenlnge, um die Sortierzeit zu verkrzen,
            ' zeichne anschlieend das Men erneut, um vorhergehende Zeit-
            ' ergebnisse zu lschen (da sie mit zuknftigen Ergebnissen
            ' nicht vergleichbar sind):
            Pause = (2 * Pause) / 3
            FensterInit

         CASE "<"

            ' Erhhe die Pausenlnge, um die Sortierzeit zu verlngern,
            ' zeichne anschlieend das Men erneut, um vorhergehende
            ' Zeitergebnisse zu lschen (da sie mit zuknftigen Ergeb-
            ' nissen nicht vergleichbar sind):
            Pause = (3 * Pause) / 2
            FensterInit

         CASE "M"
            UmschaltMus 12, SPALTELINKS + 13

         CASE Escape$

            ' Benutzer bettigte ESC, also beende diese Prozedur und gehe
            ' auf Modul-Ebene zurck:
            EXIT DO

         CASE ELSE

            ' Ungltige Taste
      END SELECT

      IF (Wahl >= 1) AND (Wahl <= ANZSORT) THEN
         Vordergrund = MaxFarben        ' Schalte invertierte Darstellung aus.
         Hintergrund = 0
         VerbrauchZeit 0                ' Gib letzte Zeit aus.
      END IF

   LOOP

END SUB

' ============================ TauscheBalken ================================
'   Ruft ZeichneBalken zweimal auf, um die Balken in Zeile1 und Zeile2 zu
'   tauschen, ruft danach die Prozedur VerbrauchZeit auf.
' ===========================================================================
'
SUB TauscheBalken (Zeile1, Zeile2) STATIC
   ZeichneBalken Zeile1
   ZeichneBalken Zeile2
   VerbrauchZeit Zeile1
END SUB

' ============================== UmschaltMus =================================
'   Kehrt den aktuellen Wert fr KeineMus um, gibt den Wert anschlieend in
'   dem Sortiermen neben der Option "Musik ein/aus" aus.
' ============================================================================
'
SUB UmschaltMus (Zeile, Spalte) STATIC
   KeineMus = NOT KeineMus
   LOCATE Zeile, Spalte
   IF KeineMus THEN
      PRINT ": AUS";
   ELSE
      PRINT ": EIN ";
   END IF
END SUB

' ============================ VerbrauchZeit ================================
'   Gibt die Sekunden aus, die seit dem Start der angegebenen Sortierroutine
'   vergangen sind.
'   Beachten Sie, da diese Zeit sowohl die Zeit zum Neuzeichnen der
'   Balken, als auch die Pause, whrend der die SOUND-Anweisung eine Note
'   spielt, beinhaltet, und daher keinen genauen Anhaltspunkt fr die
'   Sortiergeschwindigkeit darstellt.
' ============================================================================
'
SUB VerbrauchZeit (AktZeile) STATIC
   CONST FORMAT = "  &###.### sek "

   ' Gib die aktuelle Wahl sowie die Anzahl der vergangenen Sekunden in
   ' invertierter Darstellung aus:
   COLOR Vordergrund, Hintergrund
   LOCATE Wahl + 4, SPALTELINKS - 2
   PRINT USING FORMAT; OptionNamen(Wahl), TIMER - StartZeit;

   IF KeineMus THEN
      SOUND 30000, Pause            ' Musik aus, daher nur Pause.
   ELSE
      SOUND 60 * AktZeile, Pause    ' Musik ein, spiele also eine Note
   END IF                           ' whrend der Pause

   COLOR MaxFarben, 0               ' Stelle normale Vordergrund- und
                                    ' Hintergrundfarbe wieder her.
END SUB

' ============================ ZeichneBalken ===============================
'  Gibt SortDatFeld(Zeile).BalkZeichK aus auf der von dem Parameter Zeile
'  angegebenen Zeile mit der Farbe aus SortDatFeld(Zeile).FarbWert.
' ============================================================================
'
SUB ZeichneBalken (Zeile) STATIC
   LOCATE Zeile, 1
   COLOR SortDatFeld(Zeile).FarbWert
   PRINT SortDatFeld(Zeile).BalkZeichK;
END SUB

' ============================ ZeichnRahmen ==================================
'   Zeichnet mit den ASCII-Zeichen hherer Ordnung  (201),  (187) ,  (200),
'    (188) ,  (186) und  (205) einen rechteckigen Rahmen. Die Parameter
'   SeiteOben, SeiteUnten, SeiteLinks und SeiteRechts sind die Zeilen- bzw.
'   Spaltenargumente der oberen linken bzw. unteren rechten Ecke des Rahmens.
' ============================================================================
'
SUB ZeichnRahmen (SeiteOben, SeiteUnten, SeiteLinks, SeiteRechts) STATIC
   CONST OLINKS = 201, ORECHTS = 187, ULINKS = 200, URECHTS = 188
   CONST VERTIKAL = 186, HORIZONTAL = 205

   RahmenBreit = SeiteRechts - SeiteLinks - 1
   LOCATE SeiteOben, SeiteLinks
   PRINT CHR$(OLINKS); STRING$(RahmenBreit, HORIZONTAL); CHR$(ORECHTS);
   FOR Zeile = SeiteOben + 1 TO SeiteUnten - 1
      LOCATE Zeile, SeiteLinks
      PRINT CHR$(VERTIKAL); SPC(RahmenBreit); CHR$(VERTIKAL);
   NEXT Zeile
   LOCATE SeiteUnten, SeiteLinks
   PRINT CHR$(ULINKS); STRING$(RahmenBreit, HORIZONTAL); CHR$(URECHTS);
END SUB

' ============================== ZufInt% ===================================
'   Gibt eine ganze Zufallszahl zurck, die grer oder gleich dem Parameter
'   Untere und kleiner gleich dem Parameter Obere ist.
' ==========================================================================
'
FUNCTION ZufInt% (Untere, Obere) STATIC
   ZufInt% = INT(RND * (Obere - Untere + 1)) + Untere
END FUNCTION

